module PWM #
(
parameter                   N     = 24,            // length of input data 
parameter                   CYCLE = 240_000        // 240K * (1 / 12Mhz) = 20ms when 12Mhz clk_in       
)
(
input					    clk_in,			       // 12MHz system clock
input					    rst_n_in,		       // system reset, active low

input                       PWM_en,                // enable the PWM module
input          [(N-1):0]    PWM_width,             // number of high 
output  reg                 PWM_busy,              // PWM is working
output  reg                 PWM_out
);

//reg                         work;                  // control to work
reg            [(N-1):0]    num;                   // count number
reg            [(N-1):0]    width;                   // count number

// control status of work
always @ (posedge clk_in or negedge rst_n_in) begin
	if(!rst_n_in) begin
		PWM_busy <= 1'b0;
		num <= {N{1'b0}};
        width <= {N{1'b0}};
		end
	else if(PWM_en && (!PWM_busy)) begin
		PWM_busy <= 1'b1;
		width <= PWM_width;
		end
	else if(num>=CYCLE) begin
		PWM_busy <= 1'b0;
		num <= {N{1'b0}};
		end
	else if(PWM_busy) begin
		num <= num + 1'b1;
		end
end

// generate PWM
always @ (posedge clk_in or negedge rst_n_in) begin
	if(!rst_n_in) begin
		PWM_out <= 1'b0;
		end
	else  if(PWM_busy) begin
		if(num < width) begin
			PWM_out <= 1'b1;
			end
		else begin
			PWM_out <= 1'b0;
			end
		end	
	else begin
		PWM_out <= 1'b0;
		end
end

endmodule