// --------------------------------------------------------------------
// >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
// --------------------------------------------------------------------
// Module: Uart_Tx
// 
// Author: Step
// 
// Description: The transfer module of uart interface
// 
// Web: www.stepfapga.com
// 
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Version: |Mod. Date:   |Changes Made:
// V1.0     |2016/04/20   |Initial ver
// --------------------------------------------------------------------
module Uart_Tx
(
input					clk_in,			// 12MHz system clock
input					rst_n_in,		// system reset, active low

output	reg				bps_en,			// beat clock enable
input					bps_clk,		// beat clock input

input					tx_data_valid,	// tx_data_in valid when tx_data_valid active
input			[7:0]	tx_data_in,		// data need to transfer
output	reg				rs232_tx,		// uart transfer pin
output  reg             tx_busy         // uart tx busy
);

reg				[3:0]	num;
reg				[9:0]	tx_data_r;	
//start 8bit_data transfer operation
always @ (posedge clk_in or negedge rst_n_in) begin
	if(!rst_n_in) begin
		bps_en <= 1'b0;
		tx_busy <= 1'b0;
		tx_data_r <= 10'd0;
	end else if(tx_data_valid && (!bps_en))begin	
		bps_en <= 1'b1;
		tx_busy <= 1'b1;
		tx_data_r <= {1'b1,tx_data_in,1'b0};	
	end else if(num==4'd10) begin	
		bps_en <= 1'b0;
		tx_busy <= 1'b0;
	end
end

//transfer data with bps_clk when bps_en is actived
always @ (posedge clk_in or negedge rst_n_in) begin
	if(!rst_n_in) begin
		num <= 1'b0;
		rs232_tx <= 1'b1;
	end else if(bps_en) begin
		if(bps_clk) begin
			num <= num + 1'b1;
			rs232_tx <= tx_data_r[num];
		end else if(num>=4'd10) 
			num <= 4'd0;	
	end
end

endmodule
